@@ -23,6 +23,7 @@ from .encoders import NALabelEncoder
 from .encoders import RobustLabelEncoder
 from .encoders import RobustOrdinalEncoder
 from .encoders import ThresholdOneHotEncoder
+from .encoders import WOEEncoder
 
 __all__ = [
     "BaseExtremeValueTransformer",
@@ -37,4 +38,5 @@ __all__ = [
     "RobustStandardScaler",
     "log_transform",
     "quantile_transform_nonrandom",
+    "WOEEncoder",
 ]
